#TODO: 
# logger
# raw data
# 401 for specific resources
require File.expand_path('../api_methods', __FILE__)

module AdriverApi
  class Client

    attr_reader :login
    attr_reader :connection

    include ApiMethods

    def initialize(login, pass, debug: false)
      @login = login
      @password = pass
      @debug = debug
      @connection = init_connection
      refresh_token
    end

    private 

    def make_request(url, **opts)
      url = "#{URL}/#{url}"
      begin
        retries ||= 0
        response = @connection.get url, opts
      rescue ::AdriverApi::UnAuthorized => e
        refresh_token
        retry if (retries += 1) < 3
        raise e
      end
      response
    end

    def init_connection
      options = {
        :headers => {
          'Content-Type' => 'application/atom+xml',
          'X-Auth-Login' => @login.to_s,
          'X-Auth-UserID' => @user_id.to_s,
          'X-Auth-Passwd' => @token || @password.to_s
        }
        # :proxy => proxy,
      }

      Faraday::Connection.new(options) do |connection|
        connection.use Faraday::Request::UrlEncoded
        connection.use Faraday::Response::ParseXml
        connection.use FaradayMiddleware::RaiseHttpException
        connection.response :logger, @logger, :bodies => true if @debug
        connection.adapter Faraday.default_adapter
      end
    end

    def refresh_token
      url = "#{URL}/login"
      @token = nil
      @connection = init_connection
      response = @connection.get url
      @user_id = response.body["entry"]["userId"]
      @token = response.body["entry"]["token"]
      @connection = init_connection
    end
  end
end